/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.operatingsystem;

import org.junit.Test;

import java.io.File;

import static org.javafmi.kernel.CommonsKernel.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeTrue;

public class FmuFileTest {

    @Test
    public void testExtractingModelDescription() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if(systemIsWin32()) testExtractModelDescription(WINDOWS_32_BOUNCING_BALL_FMU);
        else if(systemIsLinux32()) testExtractModelDescription(LINUX_32_PLUSUN_FMU);
    }

    @Test
    public void testExtractingLibrary() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if(systemIsWin32()) testExtractLibrary(WINDOWS_32_BOUNCING_BALL_FMU, "bouncingBall", ".dll");
        else if(systemIsLinux32()) testExtractLibrary(LINUX_32_PLUSUN_FMU, "plusun", ".so");
    }

    @Test
    public void testExtractingAnFMUWithEmptyResourcesDirectory() {
        FmuFile fmu = createFmuFile(FMU_WITH_RESOURCES_BUT_EMPTY);
        File resources = fmu.getResourcesDirectory();
        File[] files = resources.listFiles();
        assertEquals(resources.getPath(), fmu.getResourcesDirectoryPath());
        assertTrue(resources.getPath().endsWith("resources"));
        assertEquals(0, files.length);
        fmu.deleteTemporal();
    }

    @Test
    public void testExtractingAnFMUWithSeveralFilesInResourcesDirectory() {
        FmuFile fmu = createFmuFile(FMU_WITH_RESOURCES);
        File resources = fmu.getResourcesDirectory();
        File[] files = resources.listFiles();
        assertEquals(resources.getPath(), fmu.getResourcesDirectoryPath());
        assertTrue(resources.getPath().endsWith("resources"));
        assertEquals(3, files.length);
        fmu.deleteTemporal();
    }


    @Test
    public void testDeletionOfTemporalFile() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if(systemIsWin32()) this.testDeletionOfTemporalFile(WINDOWS_32_BOUNCING_BALL_FMU);
        else if(systemIsLinux32()) testDeletionOfTemporalFile(LINUX_32_PLUSUN_FMU);
    }

    private void testExtractModelDescription(String fmuPath) {
        FmuFile fmuFile = createFmuFile(fmuPath);
        File modelDescriptionFile = fmuFile.getModelDescriptionFile();
        assertNotNull(modelDescriptionFile);
        assertEquals("modelDescription.xml", modelDescriptionFile.getName());
    }

    private void testDeletionOfTemporalFile(String fmuPath) {
        FmuFile fmuFile = createFmuFile(fmuPath);
        assertTrue(new File(fmuFile.getWorkingDirectoryPath()).exists());
        fmuFile.deleteTemporal();
        assertTrue(!new File(fmuFile.getWorkingDirectoryPath()).exists());
    }

    private void testExtractLibrary(String fmuPath, String modelName, String libraryExtension) {
        FmuFile fmuFile = createFmuFile(fmuPath);
        String libraryFile = fmuFile.getLibraryPath();
        assertNotNull(libraryFile);
        assertTrue(libraryFile, libraryFile.endsWith(modelName + libraryExtension));
    }
}
