/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.operatingsystem;

import org.junit.AfterClass;
import org.junit.Test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.javafmi.kernel.CommonsKernel.LINUX_32_PLUSUN_FMU;
import static org.javafmi.kernel.CommonsKernel.WINDOWS_32_BOUNCING_BALL_FMU;
import static org.javafmi.kernel.operatingsystem.SystemUtils.deleteAll;
import static org.junit.Assert.assertEquals;

public class UnZipperTest {

    private static String outPath = "tmp";
    private String sourcePath = PathBuilder.buildPath(new String[]{"../test resources", "fmu", "cosimulation"});

    @AfterClass
    public static void clean() {
        deleteAll(outPath);
    }

    @Test
    public void testUnzipModelDescription() {
        File fmuFile = getFmuFile(WINDOWS_32_BOUNCING_BALL_FMU);
        UnZipper unZipper = new UnZipper(fmuFile);
        String modelDescription = "modelDescription.xml";
        File modelDescriptionFile = unZipper.unzip("modelDescription.xml", new File(outPath));
        assertEquals("08441861ad7298c191209b764f15f2d2", calculateChecksum(modelDescriptionFile));
        assertEquals(PathBuilder.buildPath(outPath, modelDescription), modelDescriptionFile.getPath());
    }

    @Test
    public void unzipWin32Library() {
        File fmuFile = getFmuFile(WINDOWS_32_BOUNCING_BALL_FMU);
        UnZipper unZipper = new UnZipper(fmuFile);
        String entryPath = "binaries/win32";
        String extension = "dll";
        File libraryFile = unZipper.unzipFileWithExtension(entryPath, extension, new File(outPath));
        assertEquals("5269558d1cd1aa76e1be31f3cc7875a4", calculateChecksum(libraryFile));
        assertEquals(PathBuilder.buildPath(new String[]{outPath, entryPath}, "bouncingBall.dll"), libraryFile.getPath());
    }

    @Test
    public void unzipLinux32Library() {
        File fmuFile = getFmuFile(LINUX_32_PLUSUN_FMU);
        UnZipper unZipper = new UnZipper(fmuFile);
        String entryPath = "binaries/linux32";
        String extension = "so";
        File libraryFile = unZipper.unzipFileWithExtension(entryPath, extension, new File(outPath));
        assertEquals("ae7eeb6ac598a9c83c494bc146ae31a6", calculateChecksum(libraryFile));
        assertEquals(PathBuilder.buildPath(new String[]{outPath, entryPath}, "plusun.so"), libraryFile.getPath());
    }

    private File getFmuFile(String fmuFile) {
        return new File(PathBuilder.buildPath(sourcePath, fmuFile));
    }

    private String calculateChecksum(File file) {
        FileInputStream inputStream = null;
        try {
            MessageDigest messageDigest = MessageDigest.getInstance("MD5");
            inputStream = new FileInputStream(file);
            byte[] dataBytes = new byte[1024];
            int bytesRead;
            while(true) {
                if((bytesRead = inputStream.read(dataBytes)) < 0)
                    break;
                messageDigest.update(dataBytes, 0, bytesRead);
            }

            byte[] digestBytes = messageDigest.digest();
            StringBuilder stringBuffer = new StringBuilder("");
            for(int i = 0; i < digestBytes.length; i++)
                stringBuffer.append(Integer.toString((digestBytes[i] & 0xff) + 0x100, 16).substring(1));
            inputStream.close();
            return stringBuffer.toString();


        } catch(IOException | NoSuchAlgorithmException ex) {
            Logger.getLogger(UnZipperTest.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                inputStream.close();
            } catch(IOException ex) {
                Logger.getLogger(UnZipperTest.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return null;
    }
}
