/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v1;

import org.javafmi.kernel.FmuProxy;
import org.javafmi.kernel.Status;
import org.junit.Test;

import java.util.LinkedList;
import java.util.List;

import static org.javafmi.kernel.CommonsKernel.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assume.assumeTrue;

public class FmuProxyComputationTest {

    private static final double TIMEOUT = 1000.;
    private static final double START_TIME = 0.;
    private static final double STOP_TIME = 10.;

    @Test
    public void testDoStep() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if(systemIsWin32()) testDoStepsForWin32Fmus();
        else if(systemIsLinux32()) testDoStepsForLinux32Fmus();
    }

    @Test
    public void testStepCantBeCancelled() {
        assumeTrue(systemIsWin32() || systemIsLinux32());
        if(systemIsWin32()) testCancelStepsForWin32Fmus();
        else if(systemIsLinux32()) testCancelStepsForLinux32Fmus();
    }

    private void testDoStepsForWin32Fmus() {
        testDoStepForBouncingBall();
        testDoStepForIncrements();
    }

    private void testDoStepsForLinux32Fmus() {
        testDoStepForPlusun();
    }

    private void testDoStepForBouncingBall() {
        double[] height = {1.0, 0.955855, 0.813609, 0.573264, 0.234819, 0.105131};
        double stepSize = 0.1;
        FmuProxy fmuProxy = createProxy(WINDOWS_32_BOUNCING_BALL_FMU);
        fmuProxy.instantiate(TIMEOUT);
        Status initializeStatus = fmuProxy.initialize(START_TIME, STOP_TIME);
        boolean doubleValuesAreEquals = true;
        List<Status> doStepStatuses = new LinkedList<>();
        double simulationTime = 0;
        for(int i = 0; i < height.length; i++) {
            doubleValuesAreEquals &= (height[i] - fmuProxy.getReal(new int[]{0})[0] < 0.001);
            doStepStatuses.add(fmuProxy.doStep(simulationTime, stepSize));
            simulationTime += stepSize;
        }
        Status terminateStatus = fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, initializeStatus);
        assertTrue(doubleValuesAreEquals);
        for(Status doStepStatus : doStepStatuses)
            assertEquals(Status.OK, doStepStatus);
        assertEquals(Status.OK, terminateStatus);
    }

    private void testDoStepForIncrements() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_INC_FMU);
        fmuProxy.instantiate(TIMEOUT);
        Status initializeStatus = fmuProxy.initialize(START_TIME, STOP_TIME);
        List<Status> doStepStatuses = new LinkedList<>();
        double simulationTime = 0;
        for(int i = 0; i < 10; i++) {
            doStepStatuses.add(fmuProxy.doStep(simulationTime, 1));
            simulationTime += 1;
        }
        Status terminateStatus = fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, initializeStatus);
        for(Status doStepStatus : doStepStatuses) {
            assertEquals(Status.OK, doStepStatus);
        }
        assertEquals(Status.OK, terminateStatus);
    }

    private void testDoStepForPlusun() {
        FmuProxy fmuProxy = createProxy(LINUX_32_PLUSUN_FMU);
        fmuProxy.instantiate(1000.);
        Status initializeStatus = fmuProxy.initialize(START_TIME, STOP_TIME);
        double stepSize = 0.12;
        List<Status> doStepStatuses = new LinkedList<>();
        double simulationTime = 0;
        for(int i = 0; i < 1000; i++) {
            doStepStatuses.add(fmuProxy.doStep(simulationTime, stepSize));
            simulationTime += stepSize;
        }
        Status terminateStatus = fmuProxy.terminate();
        fmuProxy.freeInstance();


        assertEquals(Status.OK, initializeStatus);
        for(Status doStepStatus : doStepStatuses)
            assertEquals(Status.OK, doStepStatus);
        assertEquals(Status.OK, terminateStatus);
    }

    private void testCancelStepsForWin32Fmus() {
        testCancelStepsForBouncingBall();
    }

    private void testCancelStepsForBouncingBall() {
        double stepSize = 0.1;
        FmuProxy fmuProxy = createProxy(WINDOWS_32_BOUNCING_BALL_FMU);
        fmuProxy.instantiate(TIMEOUT);
        Status initializeStatus = fmuProxy.initialize(START_TIME, STOP_TIME);
        Status doStepStatus1 = fmuProxy.doStep(START_TIME, stepSize);
        Status cancelStepStatus = fmuProxy.cancelStep();
        Status doStepStatus2 = fmuProxy.doStep(START_TIME, stepSize);
        Status terminateStatus = fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, initializeStatus);
        assertEquals(Status.OK, doStepStatus1);
        assertEquals(Status.ERROR, cancelStepStatus);
        assertEquals(Status.OK, doStepStatus2);
        assertEquals(Status.OK, terminateStatus);
    }

    private void testCancelStepsForLinux32Fmus() {
        testCancelStepsForPlusun();
    }

    private void testCancelStepsForPlusun() {
        double timeout = 1000;
        double startTime = 0.;
        double stopTime = 5.;
        double stepSize = 1.;
        FmuProxy fmuProxy = createProxy(LINUX_32_PLUSUN_FMU);
        fmuProxy.instantiate(timeout);
        Status initializeStatus = fmuProxy.initialize(startTime, stopTime);
        Status doStepStatus1 = fmuProxy.doStep(startTime, stepSize);
        Status cancelStepStatus = fmuProxy.cancelStep();
        Status doStepStatus2 = fmuProxy.doStep(startTime, stepSize);
        Status terminateStatus = fmuProxy.terminate();

        assertEquals(Status.OK, initializeStatus);
        assertEquals(Status.OK, doStepStatus1);
        assertEquals(Status.ERROR, cancelStepStatus);
        assertEquals(Status.OK, doStepStatus2);
        assertEquals(Status.OK, terminateStatus);
        fmuProxy.freeInstance();
    }
}
