/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v1;

import org.javafmi.kernel.FmuProxy;
import org.javafmi.kernel.Status;
import org.junit.Test;

import static org.javafmi.kernel.CommonsKernel.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeTrue;

public class FmuProxyTransferOfInputOutputTest {

    @Test
    public void testGetReal() {
        assumeTrue(systemIsWin32());
        testGetRealInWin32CapableFMU();
    }

    @Test
    public void testGetInteger() {
        assumeTrue(systemIsWin32());
        testGetIntegerInWin32CapableFMU();
    }

    @Test
    public void testGetBoolean() {
        assumeTrue(systemIsWin32());
        testGetBooleanInWin32CapableFMU();
    }

    @Test
    public void testGetString() {
        assumeTrue(systemIsWin32());
        testGetStringInWin32CapableFMU();
    }

    @Test
    public void testGetEnumeration() {
        assumeTrue(systemIsWin32());
        testGetEnumerationInWin32CapableFMU();
    }

    @Test
    public void testSetReal() {
        assumeTrue(systemIsWin32());
        testSetRealInWin32CapableFMU();
    }

    @Test
    public void testSetInteger() {
        assumeTrue(systemIsWin32());
        testSetIntegerInWin32CapableFMU();
    }

    public void testSetBoolean() {
        fail(ERROR_MESSAGE_FOR_NOT_DEFINED_ARCHITECTURE);
    }

    public void testSetString() {
        fail(ERROR_MESSAGE_FOR_NOT_DEFINED_ARCHITECTURE);
    }

    private void testGetRealInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_REAL_READABLE_FMU);
        double timeout = 1000;
        double startTime = 0.;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        fmuProxy.initialize(startTime, stopTime);
        double[] realValues = fmuProxy.getReal(new int[]{0, 2, 3, 4});
        fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(1.0, realValues[0], 0.01);
        assertEquals(0.0, realValues[1], 0.01);
        assertEquals(-9.81, realValues[2], 0.01);
        assertEquals(0.7, realValues[3], 0.01);

    }

    private void testGetIntegerInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_INTEGER_READABLE_FMU);
        double timeout = 1000;
        double startTime = 0.;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        fmuProxy.initialize(startTime, stopTime);
        int integerValue = fmuProxy.getInteger(0);
        fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(1, integerValue);
    }

    private void testGetBooleanInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_BOOLEAN_READABLE_FMU);
        double timeout = 1000.;
        double startTime = 0.;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        fmuProxy.initialize(startTime, stopTime);
        boolean booleanValue = fmuProxy.getBoolean(0);
        fmuProxy.terminate();
        fmuProxy.freeInstance();
        assertTrue(booleanValue);
    }

    private void testGetStringInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_STRING_READABLE_FMU);
        double timeout = 1000.;
        double startTime = 0.;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        fmuProxy.initialize(startTime, stopTime);
        String stringValue = fmuProxy.getString(1);
        fmuProxy.terminate();
        fmuProxy.freeInstance();
        assertEquals("jan", stringValue);
    }

    private void testGetEnumerationInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_ENUMERATION_READABLE_FMU);
        double timeout = 1000;
        double startTime = 0.;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        fmuProxy.initialize(startTime, stopTime);
        Object enumerationValue = fmuProxy.getEnumeration(fmuProxy.getModelDescription().getVariable("regulation_vitesse.transferFunction.initType"));
        fmuProxy.terminate();
        fmuProxy.freeInstance();
        assertEquals("InitialOutput", enumerationValue);
    }

    private void testSetRealInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_REAL_WRITABLE_FMU);
        double timeout = 1000.;
        double startTime = 0.0;
        double stopTime = 1.0;
        fmuProxy.instantiate(timeout);
        Status setRealStatus = fmuProxy.setReal(new int[]{0}, new double[]{50.0});
        fmuProxy.initialize(startTime, stopTime);
        double realValue = fmuProxy.getReal(new int[]{0})[0];
        fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, setRealStatus);
        assertEquals(50, realValue, 0.01);
    }

    private void testSetIntegerInWin32CapableFMU() {
        FmuProxy fmuProxy = createProxy(WINDOWS_32_INTEGER_WRITABLE_FMU);
        double timeout = 1000.;
        double startTime = 0;
        double stopTime = 1.;
        fmuProxy.instantiate(timeout);
        Status setIntegerStatus = fmuProxy.setInteger(0, 5);
        fmuProxy.initialize(startTime, stopTime);
        int integerValue = fmuProxy.getInteger(0);
        fmuProxy.terminate();
        fmuProxy.freeInstance();

        assertEquals(Status.OK, setIntegerStatus);
        assertEquals(5, integerValue, 0.01);
    }
}
