/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
 package org.javafmi.kernel.v2.nativeaccess;

import com.sun.jna.Memory;
import com.sun.jna.Pointer;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static org.javafmi.kernel.Status.OK;
import static org.javafmi.kernel.Status.PENDING;
import static org.javafmi.kernel.operatingsystem.SystemUtils.*;
import static org.junit.Assert.*;

public class FmiV2Test {

    public static final boolean NO_SET_FMU_STATE_PRIOR_TO_CURRENT_POINT = false;
    private static final String LIBRARY_PATH = "../test resources/fmu/cosimulation/v2test";
    private static final String INSTANCE_NAME = "libsimplefmu";
    private static final String GUID = "simpleGuID";
    private static final String RESOURCES_LOCATION = "file://";
    private static final boolean VISIBLE = false;
    private static final boolean LOGGING_ON = false;
    private static final boolean STOP_TIME_DEFINED = true;
    private static final boolean TOLERANCE_DEFINED = false;
    private static final int DO_STEP_STATUS = 0;
    private static final int PENDING_STATUS = 1;
    private static final int LAST_SUCCESSFUL_TIME = 2;
    private static final int TERMINATED_STATUS = 3;
    private static final int ANY_STATUS = 0;
    private static Map<String, String> extensions;
    private static Map<String, String> names;
    private FmiV2 fmiV2;

    @BeforeClass
    public static void beforeClass() {
        extensions = new HashMap<>();
        extensions.put(MAC, ".dylib");
        extensions.put(WINDOWS, ".dll");
        extensions.put(LINUX, ".so");
        names = new HashMap<>();
        names.put(MAC, "libsimplefmu");
        names.put(WINDOWS, "cygsimplefmu");
        names.put(LINUX, "libsimplefmu");
    }


    @Before
    public void setUp() throws Exception {
        fmiV2 = new FmiV2(getLibraryPath());
    }

    @After
    public void clean() {
        fmiV2.free();
    }

    @Test
    public void createsAndDestroysLibrary() {
        try {
            fmiV2.instantiate(INSTANCE_NAME, FmiV2Interface.FmiType.fmiCoSimulation, GUID, RESOURCES_LOCATION, new CallbackFunctionsByReference(null), VISIBLE, LOGGING_ON);
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }
    }

    @Test
    public void setDebugLogging() {
        assertEquals(OK.getCode(), new FmiV2(getLibraryPath()).setDebugLogging(LOGGING_ON, "categories..."));
    }

    @Test
    public void makeCallsRelatedWithInitTerminateAndReset() {
        double startTime = 0.;
        double stopTime = 1.;
        double tolerance = 1E-6;
        assertEquals(OK.getCode(), fmiV2.setUpExperiment(TOLERANCE_DEFINED, tolerance, startTime, STOP_TIME_DEFINED, stopTime));
        assertEquals(OK.getCode(), fmiV2.enterInitializationMode());
        assertEquals(OK.getCode(), fmiV2.exitInitializationMode());
        assertEquals(OK.getCode(), fmiV2.terminate());
        assertEquals(OK.getCode(), fmiV2.reset());
    }

    @Test
    public void handleRealOperations() {
        int[] valueReferences = new int[]{10, 11};
        double[] doubleValues = new double[]{3.141592, 2.71828};
        assertEquals(OK.getCode(), fmiV2.setReal(valueReferences, doubleValues));
        assertEquals(doubleValues[0], fmiV2.getReal(valueReferences)[0], 0.01);
        assertEquals(doubleValues[1], fmiV2.getReal(valueReferences)[1], 0.01);
    }

    @Test
    public void handleIntegerOperations() {
        int[] valueReferences = new int[]{20, 21, 22};
        int[] integerValues = new int[]{1989, 9, 3};
        assertEquals(OK.getCode(), fmiV2.setInteger(valueReferences, integerValues));
        assertEquals(integerValues[0], fmiV2.getInteger(valueReferences)[0]);
        assertEquals(integerValues[1], fmiV2.getInteger(valueReferences)[1]);
        assertEquals(integerValues[2], fmiV2.getInteger(valueReferences)[2]);
    }

    @Test
    public void handleBooleanOperations() {
        int[] valueReferences = new int[]{30, 31, 32};
        boolean[] booleanValues = new boolean[]{false, true, false};
        assertEquals(OK.getCode(), fmiV2.setBoolean(valueReferences, booleanValues));
        assertEquals(booleanValues[0], fmiV2.getBoolean(valueReferences)[0]);
        assertEquals(booleanValues[1], fmiV2.getBoolean(valueReferences)[1]);
        assertEquals(booleanValues[2], fmiV2.getBoolean(valueReferences)[2]);
    }

    @Test
    public void handleStringOperations() {
        int[] valueReferences = new int[]{40, 41, 42};
        String[] stringValues = new String[]{"string", "variables", "can be written"};
        assertEquals(OK.getCode(), fmiV2.setString(valueReferences, stringValues));
        assertEquals(stringValues[0], fmiV2.getString(valueReferences)[0]);
        assertEquals(stringValues[1], fmiV2.getString(valueReferences)[1]);
        assertEquals(stringValues[2], fmiV2.getString(valueReferences)[2]);
    }

    @Test
    public void callsToGetStateFunctions() {
        Pointer firstState = fmiV2.getFmuState(null);
        Pointer secondState = fmiV2.getFmuState(firstState);
        assertEquals(firstState, secondState);
        assertEquals(OK.getCode(), fmiV2.setFmuState(secondState));
        assertEquals(OK.getCode(), fmiV2.freeFmuState(secondState));
    }

    @Test
    public void callsToSerializeStateFunctions() {
        Pointer fmuState = fmiV2.getFmuState(new Memory(Pointer.SIZE));
        byte[] serializedState = fmiV2.serializeFmuState(fmuState);
        assertEquals(fmiV2.serializedFmuStateSize(fmuState), serializedState.length);
        assertNotNull(fmiV2.deSerializeFMUState(serializedState));
    }

    @Test
    public void callsToGetDirectionalDerivative() {
        int[] unknowns = new int[]{1, 2};
        int[] known = new int[]{3, 4};
        double[] knownDerivatives = new double[]{11.2, 3.56};
        double[] directionalDerivative = fmiV2.getDirectionalDerivative(unknowns, known, knownDerivatives);
        assertEquals(2, directionalDerivative.length);
    }

    @Test
    public void callsToSetRelInputDerivatives() {
        int[] valueReferences = new int[]{20, 21, 22};
        int[] orders = new int[]{1, 2, 1};
        double[] values = new double[]{3.1, 0.2, 0.};
        assertEquals(OK.getCode(), fmiV2.setRealInputDerivatives(valueReferences, orders, values));
    }

    @Test
    public void callsToGetRealOutputDerivative() {
        int[] valueReferences = new int[]{20, 21, 22};
        int[] orders = new int[]{1, 2, 1};
        assertEquals(3, fmiV2.getRealOutputDerivatives(valueReferences, orders).length);
    }

    @Test
    public void callsToDoStep() {
        double currentCommunicationPoint = 1.3;
        double communicationStepSize = 0.2;
        assertEquals(OK.getCode(), fmiV2.doStep(currentCommunicationPoint, communicationStepSize, NO_SET_FMU_STATE_PRIOR_TO_CURRENT_POINT));
    }

    @Test
    public void cancelStep() {
        assertEquals(OK.getCode(), fmiV2.cancelStep());
    }

    @Test
    public void retrievesStatus() {
        assertEquals(PENDING.getCode(), fmiV2.getStatus(DO_STEP_STATUS));
        assertEquals(0., fmiV2.getRealStatus(LAST_SUCCESSFUL_TIME), 0.01);
        assertEquals(1, fmiV2.getIntegerStatus(ANY_STATUS));
        assertEquals(false, fmiV2.getBooleanStatus(TERMINATED_STATUS));
        assertEquals("fake status", fmiV2.getStringStatus(PENDING_STATUS));
    }

    private String getLibraryPath() {
        try {
            return new File(LIBRARY_PATH + "/" + names.get(getOSName()) + extensions.get(getOSName())).getCanonicalPath();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
