/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v1;

import org.javafmi.Simulation;
import org.javafmi.kernel.Status;
import org.junit.Assert;
import org.junit.Test;

import java.util.LinkedList;
import java.util.List;

import static org.javafmi.CommonsWrapper.*;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsLinux32;
import static org.javafmi.kernel.operatingsystem.SystemUtils.systemIsWin32;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeTrue;

public class SimulationTest {

    public static final double START_TIME = 0.0;
    public static final double STOP_TIME = 50.0;
    public static final double TIME_STEP = 1.0;

    @Test
    public void doubleInitShouldNotHaveEffect() {
        Simulation simulation = createSimulation(getFmuNameForThisOS());
        Status firstInitStatus = simulation.init(START_TIME);
        Status secondInitStatus = simulation.init(START_TIME);
        simulation.terminate();
        Assert.assertEquals(Status.OK, firstInitStatus);
        Assert.assertEquals(Status.WARNING, secondInitStatus);
    }

    @Test
    public void testReset() {
        Simulation simulation = createSimulation(getFmuNameForThisOS());
        Status firstInitStatus = simulation.init(START_TIME);
        Status doStepStatus = simulation.doStep(TIME_STEP);
        double firstSimulationTime = simulation.getSimulationTime();
        Status secondDoStepStatus = simulation.doStep(TIME_STEP);
        double secondSimulationTime = simulation.getSimulationTime();
        Status resetStatus = simulation.reset();
        double thirdSimulationTime = simulation.getSimulationTime();
        Status secondInit = simulation.init(START_TIME, STOP_TIME);
        double fourthSimulationTime = simulation.getSimulationTime();
        Status terminateStatus = simulation.terminate();

        assertEquals(Status.OK, firstInitStatus);
        assertEquals(Status.OK, doStepStatus);
        assertEquals(TIME_STEP, firstSimulationTime, 0.1);
        assertEquals(Status.OK, secondDoStepStatus);
        assertEquals(TIME_STEP + TIME_STEP, secondSimulationTime, 0.1);
        assertEquals(Status.OK, resetStatus);
        assertEquals(0.0, thirdSimulationTime, 0.1);
        assertEquals(Status.OK, secondInit);
        assertEquals(0.0, fourthSimulationTime, 0.1);
        assertEquals(Status.OK, terminateStatus);
    }

    @Test
    public void testCoSimulationStepsForWindowsFmu() {
        assumeTrue(systemIsWin32());
        Simulation simulation = createSimulation(WINDOWS_32_V1_FMU);
        Status initStatus = simulation.init(START_TIME, STOP_TIME);
        Status doStepStatus = simulation.doStep(TIME_STEP);
        Status writeVariableStatus = simulation.writeVariable("Pd", 0.3);
        Status doStepStatus2 = simulation.doStep(TIME_STEP);
        double pdValue = (double) simulation.readVariable("Pd").getValue();
        List<Status> doStepOnTimeStatuses = new LinkedList<>();
        for (int i = 2; i < 50; i++)
            doStepOnTimeStatuses.add(simulation.doStep(TIME_STEP));
        List<Status> doStepOutOfTimeStatuses = new LinkedList<>();
        for (int i = 0; i < 10; i++)
            doStepOutOfTimeStatuses.add(simulation.doStep(TIME_STEP));
        Status terminateStatus = simulation.terminate();

        assertEquals(Status.OK, initStatus);
        assertEquals(Status.OK, doStepStatus);
        assertEquals(Status.OK, writeVariableStatus);
        assertEquals(Status.OK, doStepStatus2);
        assertEquals(0.3, pdValue, 0.01);
        for (Status doStepOnTimeStatus : doStepOnTimeStatuses)
            assertEquals(Status.OK, doStepOnTimeStatus);
        for (Status doStepOutOfTimeStatus : doStepOutOfTimeStatuses)
            assertNotSame(Status.OK, doStepOutOfTimeStatus);
        assertEquals(Status.OK, terminateStatus);
    }

    @Test
    public void testCoSimulationStepsForLinuxFmu() {
        assumeTrue(systemIsLinux32());
        Simulation simulation = createSimulation(LINUX_32_FMU);
        Status initStatus = simulation.init(START_TIME, STOP_TIME);
        Status doStepStatus1 = simulation.doStep(TIME_STEP);
        String writableVariableName = "plusun_int_in";
        Status writeVariableStatus = simulation.writeVariable(writableVariableName, 4);
        Status doStepStatus2 = simulation.doStep(TIME_STEP);
        int variableValue = (int) simulation.readVariable(writableVariableName).getValue();
        List<Status> doStepOnTimeStatuses = new LinkedList<>();
        for (int i = 2; i < 50; i++)
            doStepOnTimeStatuses.add(simulation.doStep(TIME_STEP));
        List<Status> doStepOutOfTimeStatuses = new LinkedList<>();
        for (int i = 0; i < 10; i++)
            doStepOnTimeStatuses.add(simulation.doStep(TIME_STEP));
        Status terminateStatus = simulation.terminate();

        assertEquals(Status.OK, initStatus);
        assertEquals(Status.OK, doStepStatus1);
        assertEquals(Status.OK, writeVariableStatus);
        assertEquals(Status.OK, doStepStatus2);
        assertEquals(4, variableValue);
        for (Status doStepStatus : doStepOnTimeStatuses)
            assertEquals(Status.OK, doStepStatus);
        for (Status doStepOutOfTimeStatus : doStepOutOfTimeStatuses)
            assertTrue(doStepOutOfTimeStatus == simulation.doStep(TIME_STEP));
        assertEquals(Status.OK, terminateStatus);
    }

    @Test
    public void readEnumerationFromWin32Fmu() {
        assumeTrue(systemIsWin32());
        String enumerationVariable = "regulation_vitesse.limIntegrator.initType";
        String enumeratedExpectedValue = "InitialOutput";
        double startTime = 0.0;
        Simulation simulation = createSimulation(WINDOWS_32_ENUMERATION_READABLE_FMU);
        String enumerationValue = simulation.readVariable(enumerationVariable).getValue().toString();
        Status initStatus = simulation.init(startTime);
        String enumerationValueAfterInit = simulation.readVariable(enumerationVariable).getValue().toString();
        Status terminateStatus = simulation.terminate();

        assertEquals(enumeratedExpectedValue, enumerationValue);
        assertEquals(Status.OK, initStatus);
        assertEquals(enumeratedExpectedValue, enumerationValueAfterInit);
        assertEquals(Status.OK, terminateStatus);
    }
}
