/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v2;

import org.javafmi.Simulation;
import org.javafmi.kernel.operatingsystem.SystemUtils;
import org.javafmi.v2.Access;
import org.junit.Test;

import static org.javafmi.CommonsWrapper.WINDOWS_32_64_FMU_WITH_SEVERAL_IO;
import static org.javafmi.CommonsWrapper.createSimulation;
import static org.javafmi.kernel.operatingsystem.SystemUtils.*;
import static org.junit.Assert.assertTrue;
import static org.junit.Assume.assumeTrue;

public class PerformanceTest {

    public static final int ITERATIONS = 1000;

    @Test
    public void usingAccessBulkReadingShouldBeFasterThanSingleReading() {
        assumeTrue(systemIsWin32() || systemIsWin64());
        long singleReadingExecutionTime = measureSingleReadingWithAccess();
        long bulkReadingExecutionTime = measureBulkReadingWithAccess();
        System.out.println("Using Access");
        System.out.println("Single reading time: " + singleReadingExecutionTime);
        System.out.println("Bulk reading time: " + bulkReadingExecutionTime);
        assertTrue(bulkReadingExecutionTime < singleReadingExecutionTime);
    }

    @Test
    public void usingSimulationBulkReadingShouldBeFasterThanSingleReading() {
        assumeTrue(systemIsWin32() || systemIsWin64());
        long singleReadingExecutionTime = measureSingleReadingWithSimulation();
        long bulkReadingExecutionTime = measureBulkReadingWithSimulation();
        System.out.println("Using Simulation");
        System.out.println("Single reading time: " + singleReadingExecutionTime);
        System.out.println("Bulk reading time: " + bulkReadingExecutionTime);
        assertTrue(bulkReadingExecutionTime < singleReadingExecutionTime);
    }


    private long measureSingleReadingWithSimulation() {
        long startTime = System.currentTimeMillis();
        Simulation simulation = createSimulation(WINDOWS_32_64_FMU_WITH_SEVERAL_IO);
        String[] variables = simulation.getModelDescription().getVariablesNames();
        simulation.init(0);
        for(int i = 0; i < ITERATIONS; i++) {
            for(String variable : variables) {
                simulation.readVariable(variable);
            }
        }
        simulation.terminate();
        return System.currentTimeMillis() - startTime;

    }

    private long measureSingleReadingWithAccess() {
        long startTime = System.currentTimeMillis();
        Simulation simulation = createSimulation(WINDOWS_32_64_FMU_WITH_SEVERAL_IO);
        String[] variables = simulation.getModelDescription().getVariablesNames();
        simulation.init(0);
        org.javafmi.v2.Access access = new Access(simulation);
        for(int i = 0; i < ITERATIONS; i++) {
            for(String variable : variables) {
                access.getReal(access.getValueReference(variable));
            }
        }
        simulation.terminate();
        return System.currentTimeMillis() - startTime;
    }

    private long measureBulkReadingWithSimulation() {
        long startTime = System.currentTimeMillis();
        Simulation simulation = createSimulation(WINDOWS_32_64_FMU_WITH_SEVERAL_IO);
        String[] variables = simulation.getModelDescription().getVariablesNames();
        simulation.init(0);
        for(int i = 0; i < ITERATIONS; i++) {
            simulation.readVariables(variables);
        }
        simulation.terminate();
        return System.currentTimeMillis() - startTime;
    }

    private long measureBulkReadingWithAccess() {
        long startTime = System.currentTimeMillis();
        Simulation simulation = createSimulation(WINDOWS_32_64_FMU_WITH_SEVERAL_IO);
        String[] variables = simulation.getModelDescription().getVariablesNames();
        simulation.init(0);
        org.javafmi.v2.Access access = new Access(simulation);
        for(int i = 0; i < ITERATIONS; i++) {
            access.getReal(access.getValueReferences(variables));

        }
        simulation.terminate();
        return System.currentTimeMillis() - startTime;
    }


}
