/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.framework.fmiInterface;

import org.javafmi.kernel.Status;
import org.javafmi.modeldescription.v2.ModelDescription;
import org.javafmi.modeldescription.v2.ScalarVariable;

import static org.javafmi.exporter.framework.fmiInterface.TypeUtilities.getBooleanFromIntArray;
import static org.javafmi.exporter.framework.fmiInterface.TypeUtilities.getIntFromBooleanArray;

public abstract class FmiAccess {

    private ModelDescription modelDescription;

    public FmiAccess(ModelDescription modelDescription) {
        this.modelDescription = modelDescription;
    }

    public ModelDescription getModelDescription() {
        return modelDescription;
    }

    public void addVariable(ScalarVariable variable) {
        modelDescription.addVariable(variable);
    }

    public int fmiEnterInitializationMode() {
        return (enterInitializationMode()).getCode();
    }

    public int fmiExitInitializationMode() {
        return exitInitializationMode().getCode();
    }

    public int fmiTerminate() {
        return terminate().getCode();
    }

    public int fmiReset() {
        return reset().getCode();
    }

    public int fmiSetupExperiment(int toleranceDefined, double tolerance, double startTime, int stopTimeDefined, double stopTime) {
        return setupExperiment((toleranceDefined == 1), tolerance, startTime, (stopTimeDefined == 1), stopTime).getCode();
    }

    public double[] fmiGetReal(int valueReferences[]) {
        String[] nameVariable = getNameVariables(valueReferences.length, valueReferences);
        return getReal(nameVariable);
    }

    public int[] fmiGetInteger(int valueReferences[]) {
        String[] nameVariable = getNameVariables(valueReferences.length, valueReferences);
        return getInteger(nameVariable);
    }


    public int[] fmiGetBoolean(int valueReferences[]) {
        String[] nameVariable = getNameVariables(valueReferences.length, valueReferences);
        return getIntFromBooleanArray(getBoolean(nameVariable));
    }

    public String[] fmiGetString(int valueReferences[]) {
        String[] nameVariable = getNameVariables(valueReferences.length, valueReferences);
        return getString(nameVariable);
    }

    public int fmiSetReal(int valueReference[], double values[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        return setReal(nameVariable, values).getCode();

    }

    public int fmiSetInteger(int valueReference[], int values[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        setInteger(nameVariable, values);
        return 0;

    }

    public int fmiSetBoolean(int valueReference[], int values[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        boolean[] booleanValues = getBooleanFromIntArray(values);
        return setBoolean(nameVariable, booleanValues).getCode();
    }


    public int fmiSetString(int valueReference[], String values[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        return setString(nameVariable, values).getCode();
    }

    public int fmiDoStep(double currentComunicationTime, double stepSize, int noSetFmuFmuStatePriorToCurrentPoint) {
        return doStep(currentComunicationTime, stepSize, (noSetFmuFmuStatePriorToCurrentPoint == 1) ? true : false).getCode();
    }

    public int fmiCancelStep() {
        return cancelStep().getCode();
    }

    public double[] fmiGetRealOutputDerivatives(int valueReference[], int order[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        return getRealOutputDerivatives(nameVariable, order);
    }

    public double[] fmiGetDirectionalDerivative(int unknownReference[], int knownReferences[], double dvKnown[]) {
        String[] nameUnknowns = getNameVariables(unknownReference.length, unknownReference);
        String[] nameKnowns = getNameVariables(knownReferences.length, knownReferences);
        return getDirectionalDerivative(nameUnknowns, nameKnowns, dvKnown);
    }


    public int fmiSetRealInputDerivatives(int valueReference[], int order[], double values[]) {
        String[] nameVariable = getNameVariables(valueReference.length, valueReference);
        return setRealInputDerivatives(nameVariable, order, values).getCode();
    }

    private String[] getNameVariables(int nvr, int[] vr) {
        String[] nameVariable = new String[nvr];
        for(int i = 0; i < nvr; i++) {
            nameVariable[i] = getModelDescription().getNameFromValueReference(vr[i]);
        }
        return nameVariable;
    }

    public abstract Status setupExperiment(boolean toleranceDefined, double tolerance, double startTime, boolean stopTimeDefined, double stopTime);

    public abstract Status enterInitializationMode();

    public abstract Status exitInitializationMode();

    public abstract Status terminate();

    public abstract Status reset();

    public abstract double[] getReal(String[] nameVariable);

    public abstract int[] getInteger(String[] nameVariable);

    public abstract boolean[] getBoolean(String[] nameVariable);

    public abstract String[] getString(String[] nameVariable);

    public abstract Status setReal(String[] nameVariable, double[] value);

    public abstract Status setInteger(String[] nameVariable, int[] value);

    public abstract Status setBoolean(String[] nameVariable, boolean[] value);

    public abstract Status setString(String[] nameVariable, String[] value);

    public abstract Status doStep(double currentCommunicationPoint, double communicationStepSize, boolean newStep);

    public abstract Status cancelStep();

    public abstract double[] getDirectionalDerivative(String[] nameUnknowns, String[] nameKnowns, double[] dvKnown);

    public abstract double[] getRealOutputDerivatives(String[] nameVariable, int[] order);

    public abstract Status setRealInputDerivatives(String[] nameVariable, int[] order, double[] values);
}
