/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.exporter.fmubuilder;

import org.javafmi.fmubuilder.FmuBuilder;
import org.javafmi.kernel.operatingsystem.FileUtils;
import org.javafmi.kernel.operatingsystem.PathBuilder;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Constructor;

import static org.javafmi.exporter.fmubuilder.ModelDescriptionSerializer.serialize;
import static org.javafmi.kernel.operatingsystem.FileUtils.*;

public class ShellFmuBuilder {
    private static final String BOOT_CLASSPATH = "rt_original.jar";
    private File userJar;

    public ShellFmuBuilder(String userJar) {
        this.userJar = new File(userJar);
    }

    public static void main(String[] args) {
        if(args.length != 1) {
            System.err.println("Usage: java -jar builder name.jar");
            return;
        }
        File resultFmu = new ShellFmuBuilder(args[0]).buildFMU();
        System.out.println("Created " + resultFmu.getName());
    }

    public File buildFMU() {
        FmuBuilder builder = new FmuBuilder();
        File modelDescription = extractModelDescriptionToFile();
        builder.addModelDescription(modelDescription);
        FileUtils.deleteAll(modelDescription);
        builder.addLibrary(new ShellFmu(ShellFmu.DARWIN_64), "binaries/darwin64");
        builder.addLibrary(new ShellFmu(ShellFmu.WIN_64), "binaries/win64");
        builder.addLibrary(new ShellFmu(ShellFmu.WIN_32), "binaries/win32");
        //  builder.addLibrary(new ShellFmu(ShellFmu.LINUX_64), "binaries/linux64");
        File bootClasspath = loadResource(createTemporal(), BOOT_CLASSPATH);
        File unifiedRTJar = new JarBuilder("rt.jar").addClassesFromJar(bootClasspath, userJar).buildJar();
        builder.addFileAsResource(unifiedRTJar);
        deleteAll(unifiedRTJar);
        deleteAll(bootClasspath);
        return builder.buildFmu(extractFmuName());
    }

    private File createTemporal() {
        String systemTempDirectory = System.getProperty("java.io.tmpdir");
        File tempFolder = new File(PathBuilder.buildPath(systemTempDirectory, "rt_fmu" + System.currentTimeMillis()));
        tempFolder.mkdirs();
        tempFolder.deleteOnExit();
        return tempFolder;
    }

    public File extractModelDescriptionToFile() {
        return serialize(instantiateModelDescription(userJar), new File("./modelDescription.xml"));
    }

    public File loadResource(File workingDir, String resourceName) {
        InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(resourceName);
        File resourceFile = createFile(workingDir, resourceName);
        OutputStream outputStream = fileOutputStream(resourceFile);
        copyFromTo(resourceAsStream, outputStream);
        close(outputStream);
        return resourceFile;
    }

    public Object instantiateModelDescription(File finalClasspath) {
        try {
            Constructor fmiModelDescription = new JarClassLoader(new File(finalClasspath.getPath())).loadClass("FmiModelDescription").getConstructor(String.class);
            return fmiModelDescription.newInstance(extractFmuName());
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
    }

    private String extractFmuName() {
        String userJarName = userJar.getName();
        return userJarName.substring(userJarName.lastIndexOf("/") + 1, userJarName.lastIndexOf("."));
    }
}
