#include <stdlib.h>
#include <stdio.h>
#include "VirtualMachine.h"
#include "FunctionsNames.h"
#include "../Logger.h"

struct VirtualMachine{
	JavaVM* vm;
	JNIEnv* environment;
	jclass mainObjectClass;
	jobject mainObject;
};

void exitWithError(const char* errorMessage, int errorCode){
	fprintf(stderr, "%s\n", errorMessage);	
	exit(errorCode);
}

void checkForExceptions(struct VirtualMachine* self, const char* message, const char* value = "\n"){
	if(self->environment->ExceptionCheck()){
		self->environment->ExceptionDescribe();
		self->vm->DestroyJavaVM();
		fprintf(stderr, "%s%s?\n", message, value);
		exit(1);
	}
}

jintArray toJavaIntArray(struct VirtualMachine* self, int array[], int numberOfElements){
	log("converting int[] to jintArray of "); logNumber(numberOfElements);
	jintArray values = self->environment->NewIntArray(numberOfElements);
	self->environment->SetIntArrayRegion(values, 0, numberOfElements, (jint*) array);
	return values;
}

jdoubleArray toJavaDoubleArray(struct VirtualMachine* self, double array[], int numberOfElements){
	jdoubleArray values = self->environment->NewDoubleArray(numberOfElements);
	self->environment->SetDoubleArrayRegion(values, 0, numberOfElements, (jdouble*) array);
	return values;
}

jobjectArray toJavaStringArray(struct VirtualMachine* self, char* array[], int numberOfElements){
	jobjectArray values = (jobjectArray) self->environment->NewObjectArray(numberOfElements, self->environment->FindClass("java/lang/String"), self->environment->NewStringUTF(""));
    int i = 0;
    for (i = 0; i < numberOfElements; i++){
    	 self->environment->SetObjectArrayElement(values, i, self->environment->NewStringUTF(array[i]));
    }
	return values;
}

void* createVirtualMachine(char* bootstrap, char* bootClasspath){
	log("\n###########################################################");
	struct VirtualMachine* virtualMachine = (struct VirtualMachine*) calloc(1, sizeof(VirtualMachine));
	log("allocated the VM");
	log(bootstrap);
	log(bootClasspath);
	JavaVMInitArgs	vmArgs;
	vmArgs.version = JNI_VERSION_1_6;
	vmArgs.nOptions = 2;
	vmArgs.ignoreUnrecognized = JNI_TRUE;
	JavaVMOption options[vmArgs.nOptions];
	options[0].optionString = bootClasspath;
	options[1].optionString = bootstrap;
	vmArgs.options = options;
	int creationCode;
	if((creationCode = JNI_CreateJavaVM(&virtualMachine->vm, (void**)&virtualMachine->environment, &vmArgs)) < 0) {
		exitWithError("impossible to create VM", creationCode);
	}
	log("created the VM");
	return virtualMachine;
}

void destroyVirtualMachine(struct VirtualMachine* _self){
	_self->vm->DestroyJavaVM();
	//free(self);
	_self = NULL;	
}

int invokeTerminate(void* _self){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;	
	jmethodID terminateMethod = self->environment->GetMethodID(self->mainObjectClass, TERMINATE, TERMINATE_SIGNATURE);
	if(terminateMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, terminateMethod);
	//destroyVirtualMachine(self);
	return (int)jintStatus;
}


int loadMainObject(void* _self, const char* objectClassName){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jclass objectClass = self->environment->FindClass(objectClassName);
	if(objectClass == 0) checkForExceptions(self, "Is your class name ", objectClassName);
	log("class found");
	jmethodID constructorID = self->environment->GetMethodID(objectClass, "<init>", "()V");
	log("constructor found");
	if(constructorID == 0) checkForExceptions(self, "GetMethodID");
	jobject mainObject = self->environment->NewObject(objectClass, constructorID);
	if(mainObject == 0) checkForExceptions(self, "NewObject");
	log("called constructor");
	self->mainObject = mainObject;
	self->mainObjectClass = objectClass;
	return 0;
}

int invokeEnterInitializationMode(void* _self){
   	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID enterInitMethod = self->environment->GetMethodID(self->mainObjectClass, ENTER_INIT, ENTER_INIT_SIGNATURE);
	if(enterInitMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, enterInitMethod);
	return (int)jintStatus;
}

int invokeExitInitializationMode(void* _self){
   	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID exitInitMethod = self->environment->GetMethodID(self->mainObjectClass, EXIT_INIT, EXIT_INIT_SIGNATURE);
	if(exitInitMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, exitInitMethod);
	return (int)jintStatus;
}

int invokeSetupExperiment(void* _self, int toleranceDefined, double tolerance, double startTime, int stopTimeDefined, double stopTime){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setupExperimentMethod = self->environment->GetMethodID(self->mainObjectClass, SETUP_EXPERIMENT, SETUP_EXPERIMENT_SIGNATURE);
	if(setupExperimentMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setupExperimentMethod, toleranceDefined, tolerance, startTime, stopTimeDefined, stopTime);
	return (int)jintStatus;
}

int resetVirtualMachine(void* _self){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID resetMethod = self->environment->GetMethodID(self->mainObjectClass, RESET, RESET_SIGNATURE);
	if(resetMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, resetMethod);
	return (int)jintStatus;
}

void invokeGetReal(void* _self, int valueReferences[], int numberOfValues, double values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getRealMethod = self->environment->GetMethodID(self->mainObjectClass, GET_REAL, GET_REAL_SIGNATURE);
	if(getRealMethod == 0) checkForExceptions(self, "GetMethodID");
	log("found method fmiGetReal");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jdoubleArray doubleArrayValues = (jdoubleArray) self->environment->CallObjectMethod(self->mainObject, getRealMethod, javaValueReferences);
	log("called fmiGetReal, converting values to double[]");
	jdouble* doubleValues = self->environment->GetDoubleArrayElements(doubleArrayValues, 0);
	log("converted jdoubleArray to jdouble*, storing in result[]");
	for (int i = 0; i < numberOfValues; i++) {
		values[i] = (double) doubleValues[i];
	}
}


void invokeGetInteger(void* _self, int valueReferences[], int numberOfValues, int values[]){
    struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getIntMethod = self->environment->GetMethodID(self->mainObjectClass, GET_INTEGER, GET_INTEGER_SIGNATURE);
	if(getIntMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jintArray intArrayValues = (jintArray) self->environment->CallObjectMethod(self->mainObject, getIntMethod, javaValueReferences);
	jint* intValues = self->environment->GetIntArrayElements(intArrayValues, 0);
	for (int i = 0; i < numberOfValues; i++) {
		values[i] = (int) intValues[i];
	}

}

void invokeGetBoolean(void* _self, int valueReferences[], int numberOfValues, int values[]){
    struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getBooleanMethod = self->environment->GetMethodID(self->mainObjectClass, GET_BOOLEAN, GET_BOOLEAN_SIGNATURE);
	if(getBooleanMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jintArray intArrayValues = (jintArray) self->environment->CallObjectMethod(self->mainObject, getBooleanMethod, javaValueReferences);
	jint* intValues = self->environment->GetIntArrayElements(intArrayValues, 0);
	for (int i = 0; i < numberOfValues; i++) {
		values[i] = (int) intValues[i];
	}
}

void invokeGetString(void* _self, int valueReferences[], int numberOfValues, char* values[]){
    struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getStringMethod = self->environment->GetMethodID(self->mainObjectClass, GET_STRING, GET_STRING_SIGNATURE);
	if(getStringMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jobjectArray stringArrayValues = (jobjectArray) self->environment->CallObjectMethod(self->mainObject, getStringMethod, javaValueReferences);
	int stringCount = self->environment->GetArrayLength(stringArrayValues);
	for (int i=0; i<stringCount; i++) {
 		jstring string = (jstring) self->environment->GetObjectArrayElement(stringArrayValues, i);
        char *rawString = (char*)self->environment->GetStringUTFChars(string, 0);
        values[i] = rawString;
	}
}


int invokeSetReal(void* _self, int valueReferences[], int numberOfValues, double values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setRealMethod = self->environment->GetMethodID(self->mainObjectClass, SET_REAL, SET_REAL_SIGNATURE);
	if(setRealMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jdoubleArray settingValues = toJavaDoubleArray(self, values, numberOfValues);
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setRealMethod, javaValueReferences, settingValues);
	return (int)jintStatus;
}

int invokeSetInteger(void* _self, int valueReferences[], int numberOfValues, int values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setIntMethod = self->environment->GetMethodID(self->mainObjectClass, SET_INTEGER, SET_INTEGER_SIGNATURE);
	if(setIntMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jintArray settingValues = toJavaIntArray(self, values, numberOfValues);
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setIntMethod, javaValueReferences, settingValues);
	return (int)jintStatus;
}

int invokeSetBoolean(void* _self, int valueReferences[], int numberOfValues, int values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setBooleanMethod = self->environment->GetMethodID(self->mainObjectClass, SET_BOOLEAN, SET_BOOLEAN_SIGNATURE);
	if(setBooleanMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jintArray settingValues = toJavaIntArray(self, values, numberOfValues);
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setBooleanMethod, javaValueReferences, settingValues);
	return (int)jintStatus;
}

int invokeSetString(void* _self, int valueReferences[], int numberOfValues, char* values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setStringMethod = self->environment->GetMethodID(self->mainObjectClass, SET_STRING, SET_STRING_SIGNATURE);
	if(setStringMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jobjectArray settingValues = toJavaStringArray(self, values, numberOfValues);
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setStringMethod, javaValueReferences, settingValues);
	return (int)jintStatus;
}

int invokeDoStep(void* _self, double currentComunicationTime, double stepSize, int noSetFmuFmuStatePriorToCurrentPoint){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID doStepMethod = self->environment->GetMethodID(self->mainObjectClass, DO_STEP, DO_STEP_SIGNATURE);
	if(doStepMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, doStepMethod, currentComunicationTime, stepSize, noSetFmuFmuStatePriorToCurrentPoint);
	return (int)jintStatus;
}


int invokeCancelStep(void* _self){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID cancelStepMethod = self->environment->GetMethodID(self->mainObjectClass, CANCEL_STEP, CANCEL_STEP_SIGNATURE);
	if(cancelStepMethod == 0) checkForExceptions(self, "GetMethodID");
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, cancelStepMethod);
	return (int)jintStatus;
}

void invokeGetDirectionalDerivative(void* _self, int unknownReference[], int numberOfUnknowns, int knownReferences[], int numberOfKnowns, double dvKnown[], double dvUnKnown[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getDirectionalDerivativesMethod = self->environment->GetMethodID(self->mainObjectClass, GET_DIRECTIONAL_DERIVATIVES, GET_DIRECTIONAL_DERIVATIVES_SIGNATURE);
	if(getDirectionalDerivativesMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaUnknownReferences = toJavaIntArray(self, unknownReference, numberOfUnknowns);
	jintArray javaKnownReferences = toJavaIntArray(self, knownReferences, numberOfKnowns);
	jdoubleArray dvknownsValues = toJavaDoubleArray(self, dvKnown, numberOfKnowns);
	jdoubleArray doubleArrayValues = (jdoubleArray) self->environment->CallObjectMethod(self->mainObject, getDirectionalDerivativesMethod, javaUnknownReferences, javaKnownReferences, dvknownsValues);
	jdouble* doubleValues = self->environment->GetDoubleArrayElements(doubleArrayValues, 0);
	for (int i = 0; i < numberOfUnknowns; i++) {
		dvUnKnown[i] = (double) doubleValues[i];
	}	
}


void invokeGetRealOutputDerivatives(void* _self, int valueReference[], int numberOfValues, int order[], double values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID getRealOutDerivativesMethod = self->environment->GetMethodID(self->mainObjectClass, GET_REAL_OUTPUT_DERIVATIVES, GET_REAL_OUTPUT_DERIVATIVES_SIGNATURE);
	if(getRealOutDerivativesMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReference, numberOfValues);
	jintArray orderValues = toJavaIntArray(self, order, numberOfValues);
	jdoubleArray doubleArrayValues = (jdoubleArray) self->environment->CallObjectMethod(self->mainObject, getRealOutDerivativesMethod, javaValueReferences, orderValues);
	jdouble* doubleValues = self->environment->GetDoubleArrayElements(doubleArrayValues, 0);
	for (int i = 0; i < numberOfValues; i++) {
		values[i] = (double) doubleValues[i];
	}	
}

int invokeSetRealInputDerivatives(void* _self, int valueReferences[], int numberOfValues, int order[], double values[]){
	struct VirtualMachine* self = (struct VirtualMachine*) _self;
	jmethodID setRealMethod = self->environment->GetMethodID(self->mainObjectClass, SET_REAL_INPUT_DERIVATIVES, SET_REAL_INPUT_DERIVATIVES_SIGNATURE);
	if(setRealMethod == 0) checkForExceptions(self, "GetMethodID");
	jintArray javaValueReferences = toJavaIntArray(self, valueReferences, numberOfValues);
	jintArray orderValues = toJavaIntArray(self, order, numberOfValues);
	jdoubleArray settingValues = toJavaDoubleArray(self, values, numberOfValues);
	jint jintStatus = (jint) self->environment->CallIntMethod(self->mainObject, setRealMethod, javaValueReferences, orderValues, settingValues);
	return (int)jintStatus;
}



